import os
from typing import AnyStr, NoReturn, Union, Dict
from abc import ABC, abstractmethod
from rich import print # noqa

import psutil

DEBUG = os.getenv('DEBUG', False)


class Sensor(ABC):
    __doc__ = """
		Basic class defining the fundamental behavior of a EcoTrust Sensor. 	
	"""

    @abstractmethod
    def start_scan(self) -> bool:
        """
            Call /startscan on Sensor API
        :return: true|false: Is the scan accepeted ?
        """
        pass

    @abstractmethod
    def stop_scan(self) -> bool:
        """
            Call /stop/<scan_id> on Sensor API
        :return: true|false: Is the scan stopped ?
        """
        pass

    @abstractmethod
    def wait_scan(self) -> bool:
        """
            Call /status/<scan_id> periodically on Sensor API to check if the Scan is finished and/or with error.
        :return: Is scan finished ? (without error)
        """
        pass

    @abstractmethod
    def wait_engine(self) -> NoReturn:
        """
            Normaly called when the container is started, to check if the engine is ready for Scans.
        :return:
        """
        pass

    # noinspection PyMethodMayBeStatic
    def is_engine_process_running(self, engine_proc_name: Union[AnyStr, None]) -> bool:
        """
            Checks if the engine process of the Sensor is still running in background.
        :return: true|false
        """
        # Status vars
        ENGINE_NOT_RUNNING = False
        ENGINE_RUNNING = True  # noqa

        for proc in psutil.process_iter(['name']):
            if proc.info['name'] == engine_proc_name or engine_proc_name in proc.info['name']:  # noqa
                if DEBUG:
                    print(f'[bold green] Proc: {proc.info["name"]} mathes {engine_proc_name} [/bold green]')
                break

        else:
            # No process found
            return ENGINE_NOT_RUNNING

        # Process found
        return ENGINE_RUNNING

    @abstractmethod
    def wait_report(self) -> NoReturn:
        """
            Wait json/xml/csv Scan report to be available to be sent.
        :return:
        """
        pass

    @abstractmethod
    def get_report(self) -> (bool, Dict):
        """
            Gets the CSV/JSON... report from the Sensor after the scan finishes (or should be called after it).
        :return:
        """
        pass
